/*
 * Grep.java
 *
 * version: 15th December 2008, 12:05
 * author: yanas
 */
package cz.yanas.pb162.grep;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;

import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

/**
 * A simplified grep.
 *
 * @author yanas
 */
public final class Grep {

    /** the encoding of the text files */
    private static final String ENCODING = "UTF-8";

    /**
     * The entry point of the Grep application.
     *
     * @param args command line arguments
     */
    public static void main(String[] args) {
        // two arguments are required -- a regular expression and a path to a text file
        if (args.length != 2) {
            System.err.println("Invalid number of arguments!");
            System.exit(-1);
        }

        // prepare variable for the regular expression pattern
        Pattern regexp = null;

        try {
            // try to compile the regular expression to a pattern
            regexp = Pattern.compile(args[0]);
        } catch (PatternSyntaxException exception) {
            System.err.println(exception.getMessage());
            System.exit(-2);
        }

        // prepare a variable for a buffered reader
        BufferedReader bufferedReader = null;

        try {
            // try to create a buffered reader with specified encoding
            bufferedReader = new BufferedReader(new InputStreamReader(
                    new FileInputStream(args[1]), ENCODING));

            // read the first line
            String line = bufferedReader.readLine();

            // read all the lines in the text file
            while (line != null) {
                // if the current line matches the pattern
                if (regexp.matcher(line).matches()) {
                    // simply print it out to the standard output
                    System.out.println(line);
                }

                // read another line from the input text file
                line = bufferedReader.readLine();
            }
        } catch (UnsupportedEncodingException exception) {
            System.err.println(exception.getMessage());
            System.exit(-3);
        } catch (FileNotFoundException exception) {
            System.err.println(exception.getMessage());
            System.exit(-4);
        } catch (IOException exception) {
            System.err.println(exception.getMessage());
            System.exit(-5);
        } finally {
            // if the buffered reader was successfully opened
            if (bufferedReader != null) {
                try {
                    // try to close it
                    bufferedReader.close();
                } catch (IOException exception) {
                    exception.printStackTrace();
                }
            }
        }
    }

    private Grep() {
    }

}
